# This file can be used, modified and redistributed under the
# terms of the GNU Library General Public License 

"A base class for ZClasses implementing SessionDataContainers"

try:
    from Products.CoreSessionTracking.SessioningInterfaces \
         import SessionDataContainerInterface
except:
    # Up to version 0.8, CoreSessionTracking does not export interfaces
    class SessionDataContainerInterface:
        pass    

from cPickle import dumps, loads
from Shared.DC.ZRDB.TM import TM

__debug__ = 1

    
class SessionDataContainerBase(TM):
    "This class enables child ZClasses to implement the Sessiong Interface"

    # One thing to do is declare the implementation
    __implements__ = SessionDataContainerInterface

    # This code defines a function _debug in the class.
    if __debug__:
        x = open('/tmp/debug-sessions.txt','at')
        _debug = lambda s,x=x: x.write("%s\n" % s)
        del x
    else:
        _debug = lambda s: None

    # We need this cache handling
    def _init(self):
        self._register()
        cache = self._cache = getattr(self, '_cache', {})
        return cache

    def _begin(self):
        pass

    def _abort(self):
        try: del self._cache
        except: pass

    def _finish(self):
        # First approximation: Assume it changed.
        cache = self._init() # just in case...
        try:
            for k,v in cache.items():
                self[k]=v
        except:
            # So the session couldn't be saved. Tough.
            pass
        del self._cache
        
    # Another thing to do is to translate magic names to names one
    # can put in a ZClass
    def __delitem__(self, key):
        cache = self._init()
        try:
            del cache[key]
        except KeyError:
            pass
        return self.delitem(key)

    def __getitem__(self, key):
        # Caching is more for saving then for caching...
        cache = self._init()
        if cache.has_key(key):
            return cache[key]
            
        cache[key] = obj = self.getitem(key)
        
        return obj

    def __len__(self):
        self._init()
        return self.getLen()

    def __setitem__(self, key, value):
        # Handle the cache...
        cache = self._init()
        cache[key] = value
        # do the work
        return self.setitem(key, value)

    # These are default implementations
    #
    def set(self, key, value):
        return self.__setitem__(key,value)
    
    def get(self, key, default=None):
        try:
            return self[key]
        except KeyError:
            return default
    # And these are there so that formally, we implement the interface.
    # They need to be overridden
    def getLen(self):
        raise "Not implemented"

    def has_key(self, key):
        raise "Not implemented"

    # These are help functions
    #
    def pickle(self, object):
        return dumps(object)

    def unpickle (self, string):
        return loads(string)

    # Timeout is part of the container definition though not interface...
    def setTimeoutMinutes(self, minutes):
        self._timeout = minutes

    def getTimeoutMinutes(self):
        try:
            return self._timeout
        except:
            return self.getDefaultTimeoutMinutes()

    def getDefaultTimeoutMinutes(self):
        "Defined as a function to be overridable"
        return 20
        
        
    
# A class inheriting this one needs to implement:
# getLen
# has_key
# getitem (magic method for "self[key]")
# setitem (magic method for "self[key]=value")
# delitem (magic method for "del self[key]")
